@@ -12,6 +12,9 @@ import unicodedata
 import logging as lg
 import datetime as dt
 import networkx as nx
+import numpy as np
+import pandas as pd
+import requests
 
 from . import globals
 
@@ -233,5 +236,114 @@ def get_largest_component(G, strongly=False):
             log('Graph was not connected, retained only the largest weakly connected component ({:,} of {:,} total nodes) in {:.2f} seconds'.format(len(list(G.nodes())), original_len, time.time()-start_time))
 
     return G
+
     
-    
\ No newline at end of file
+def great_circle_vec(lat1, lng1, lat2, lng2, earth_radius=6371009):
+    """
+    Vectorized function to calculate the great-circle distance between two points or between vectors of points.
+
+    Parameters
+    ----------
+    lat1 : float or array of float
+    lng1 : float or array of float
+    lat2 : float or array of float
+    lng2 : float or array of float
+    earth_radius : numeric
+        radius of earth in units in which distance will be returned (default is meters)
+    
+    Returns
+    -------
+    distance : float or array of float
+        distance or vector of distances from (lat1, lng1) to (lat2, lng2) in units of earth_radius
+    """
+    
+    phi1 = np.deg2rad(90 - lat1)
+    phi2 = np.deg2rad(90 - lat2)
+    
+    theta1 = np.deg2rad(lng1)
+    theta2 = np.deg2rad(lng2)
+    
+    cos = (np.sin(phi1) * np.sin(phi2) * np.cos(theta1 - theta2) + np.cos(phi1) * np.cos(phi2))
+    arc = np.arccos(cos)
+    
+    # return distance in units of earth_radius
+    distance = arc * earth_radius
+    return distance
+    
+    
+def get_nearest_node(G, point, return_dist=False):
+    """
+    Return the graph node nearest to some specified point.
+    
+    Parameters
+    ----------
+    G : networkx multidigraph
+    point : tuple
+        the (lat, lon) point for which we will find the nearest node in the graph
+    return_dist : bool
+        optionally also return the distance between the point and the nearest node
+    
+    Returns
+    -------
+    networkx multidigraph or tuple
+        multidigraph or optionally (multidigraph, float)
+    """    
+    start_time = time.time()
+    
+    # dump graph node coordinates into a pandas dataframe indexed by node id with x and y columns
+    coords = np.array([[node, data['x'], data['y']] for node, data in G.nodes(data=True)])
+    df = pd.DataFrame(coords, columns=['node', 'x', 'y']).set_index('node')
+    
+    # add columns to the dataframe representing the (constant) coordinates of the reference point
+    df['reference_y'] = point[0]
+    df['reference_x'] = point[1]
+    
+    # calculate the distance between each node and the reference point
+    distances = great_circle_vec(lat1=df['reference_y'], 
+                                 lng1=df['reference_x'],
+                                 lat2=df['y'], 
+                                 lng2=df['x'])
+    
+    # nearest node's ID is the index label of the minimum distance
+    nearest_node = int(distances.idxmin())
+    log('Found nearest node ({}) to point {} in {:,.2f} seconds'.format(nearest_node, point, time.time()-start_time))
+    
+    # if caller requested return_dist, return distance between the point and the nearest node as well
+    if return_dist:
+        return nearest_node, distances.loc[nearest_node]
+    else:
+        return nearest_node
+        
+
+def geocode(query):
+    """
+    Geocode a query string to (lat, lon) with the Nominatim geocoder.
+
+    Parameters
+    ----------
+    query : string
+        the query string to geocode
+
+    Returns
+    -------
+    point : tuple
+        the (lat, lon) coordinates returned by the geocoder
+    """
+    
+    # send the query to the nominatim geocoder and parse the json response
+    url_template = 'https://nominatim.openstreetmap.org/search?format=json&limit=1&q={}'
+    url = url_template.format(query)
+    response = requests.get(url, timeout=60)
+    results = response.json()
+    
+    # if results were returned, parse lat and long out of the result
+    if len(results) > 0 and 'lat' in results[0] and 'lon' in results[0]:
+        lat = float(results[0]['lat'])
+        lon = float(results[0]['lon'])
+        point = (lat, lon)
+        log('Geocoded "{}" to {}'.format(query, point))
+        return point
+    else:
+        raise Exception('Nominatim geocoder returned no results for query "{}"'.format(query))
+
+        
\ No newline at end of file
